/*
	ncbricks - ncurses based brick breaker
	Copyright (C) 2017-2018  Aron Barath

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
	02110-1301  USA
*/

#include "ncbricks.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>

void ball_show_all()
{
	size_t i;

	for(i=0;i<g_balls.size;++i)
	{
		ball_show((struct ball*)g_balls.pointers[i]);
	}
}

void ball_hide_all()
{
	size_t i;

	for(i=0;i<g_balls.size;++i)
	{
		ball_hide((struct ball*)g_balls.pointers[i]);
	}
}

static const char ball_style[] = "*o";

void ball_show(const struct ball* const ball)
{
	const int cpair = (0==g_atomicball) ? CPAIR_BALL : CPAIR_BALL2;

	if(g_use_colors)
	{
		wattron(g_game_win, COLOR_PAIR(cpair));
	}

	mvwaddch(g_game_win, g_board_top+ball->dpos.y, g_board_left+ball->dpos.x, ball_style[!g_atomicball]|A_BOLD);

	if(g_use_colors)
	{
		wattroff(g_game_win, COLOR_PAIR(cpair));
	}
}

void ball_hide(const struct ball* const ball)
{
	mvwaddch(g_game_win, g_board_top+ball->dpos.y, g_board_left+ball->dpos.x, ' ');
}

void ball_delete_all()
{
	while(0<g_balls.size)
	{
		struct ball* const ball = (struct ball*)g_balls.pointers[--g_balls.size];
		ball_hide(ball);
		free(ball);
	}
}

void ball_spawn()
{
	struct ball* const ball = (struct ball*)malloc(sizeof(struct ball));
	assert(NULL!=ball);
	memset(ball, 0, sizeof(*ball));

#define BALL_SPAWN_MAGIC 1

	vec2_random(&ball->velo, BALL_VELOCITY);
	if(0<ball->velo.y) { ball->velo.y = -ball->velo.y; }
	ball->velo.y -= BALL_SPAWN_MAGIC * BALL_VELOCITY;
	vec2_sdiv(&ball->velo, &ball->velo, 1+BALL_SPAWN_MAGIC);

	ball->lpos.x = (LOGICAL_SCALE * (g_board_width - 1)) / 2;
	ball->lpos.y = LOGICAL_SCALE * (g_board_height - 3);
	ball->dpos.x = ball->lpos.x / LOGICAL_SCALE;
	ball->dpos.y = ball->lpos.y / LOGICAL_SCALE;

	ptrarr_add(&g_balls, ball);
}

void ball_split_all()
{
	size_t i;
	size_t n = g_balls.size;

	for(i=0;i<n && g_balls.size<MAX_BALLS;++i)
	{
		struct ball* const old_ball = (struct ball*)g_balls.pointers[i];

		struct ball* const ball = (struct ball*)malloc(sizeof(struct ball));
		assert(NULL!=ball);
		memset(ball, 0, sizeof(*ball));

		ball->lpos.x = old_ball->lpos.x;
		ball->lpos.y = old_ball->lpos.y;
		ball->velo.x = -old_ball->velo.y;
		ball->velo.y = old_ball->velo.x;

		if(0<ball->velo.y)
		{
			ball->velo.y = -ball->velo.y;
		}

		/* adjust the velocity of the new ball towards up direction */
		ball->velo.y -= (BALL_VELOCITY + 1);
		vec2_sdiv(&ball->velo, &ball->velo, 2);

		ball->dpos.x = ball->lpos.x / LOGICAL_SCALE;
		ball->dpos.y = ball->lpos.y / LOGICAL_SCALE;

		ptrarr_add(&g_balls, ball);
	}
}

void ball_reflect(struct ball* const ball, const struct rect2* const rect)
{
	ball->lpos.y = rect->pos.y*2-4-ball->lpos.y;
	ball->velo.y = -ball->velo.y;
	ball->hits   = 0;

	// TODO: adjust ball velocity based on the paddle
}
