/*
	ncbricks - ncurses based brick breaker
	Copyright (C) 2017-2018  Aron Barath

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
	02110-1301  USA
*/

#include "ncbricks.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>

void missile_show_all()
{
	size_t i;

	for(i=0;i<g_missiles.size;++i)
	{
		missile_show((struct missile*)g_missiles.pointers[i]);
	}
}

void missile_hide_all()
{
	size_t i;

	for(i=0;i<g_missiles.size;++i)
	{
		missile_hide((struct missile*)g_missiles.pointers[i]);
	}
}

void missile_show(const struct missile* const missile)
{
	if(g_use_colors)
	{
		wattron(g_game_win, COLOR_PAIR(CPAIR_MISSILE));
	}

	mvwaddch(g_game_win, g_board_top+missile->pos.y, g_board_left+missile->pos.x, '!'|A_BOLD);

	if(g_use_colors)
	{
		wattroff(g_game_win, COLOR_PAIR(CPAIR_MISSILE));
	}
}

void missile_hide(const struct missile* const missile)
{
	mvwaddch(g_game_win, g_board_top+missile->pos.y, g_board_left+missile->pos.x, ' ');
}

void missile_delete_all()
{
	while(0<g_missiles.size)
	{
		struct missile* const missile = (struct missile*)g_missiles.pointers[--g_missiles.size];
		missile_hide(missile);
		free(missile);
	}
}

static void spawn(const int x, const int y)
{
	struct missile* const missile = (struct missile*)malloc(sizeof(struct missile));
	assert(NULL!=missile);
	memset(missile, 0, sizeof(struct missile));

	missile->pos.x = x;
	missile->pos.y = y;
	ptrarr_add(&g_missiles, missile);
	missile_show(missile);
}

void missile_spawn(const struct paddle* const paddle)
{
	spawn(paddle->x + (paddle->width / 2), g_paddle_y - 1);
}

void missile_spawn_apocalypse()
{
	size_t i;

	for(i=0;i<BRICKS_PER_ROW;++i)
	{
		spawn((i*BRICK_WIDTH)+(BRICK_WIDTH/2), g_paddle_y - 1);
	}
}
